=== modified file 'NEWS.developers'
--- NEWS.developers	2005-10-06 06:31:16 +0000
+++ NEWS.developers	2005-10-13 07:30:57 +0000
@@ -1,4 +1,9 @@
-changes in weave branch:
+DEVELOPMENT HEAD
+
+ * Properties can be set on Revisions by passing 'revprops=' to
+   Branch.commit(), and retrieved by looking at Revision.properties.
+
+bzr 0.1
 
  * Remove RevisionReference class; Revision.parent_ids is now simply a
    list of their ids and parent_sha1s is a list of their corresponding

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2005-10-14 01:56:08 +0000
+++ bzrlib/commit.py	2005-10-15 11:38:29 +0000
@@ -160,7 +160,8 @@
                specific_files=None,
                rev_id=None,
                allow_pointless=True,
-               verbose=False):
+               verbose=False,
+               revprops=None):
         """Commit working copy as a new revision.
 
         timestamp -- if not None, seconds-since-epoch for a
@@ -176,6 +177,8 @@
 
         allow_pointless -- If true (default), commit even if nothing
             has changed and no merges are recorded.
+
+        revprops -- Properties for new revision
         """
         mutter('preparing to commit')
 
@@ -184,6 +187,7 @@
         self.rev_id = rev_id
         self.specific_files = specific_files
         self.allow_pointless = allow_pointless
+        self.revprops = revprops
 
         if timestamp is None:
             self.timestamp = time.time()
@@ -304,7 +308,8 @@
                             committer=self.committer,
                             message=self.message,
                             inventory_sha1=self.inv_sha1,
-                            revision_id=self.rev_id)
+                            revision_id=self.rev_id,
+                            properties=self.revprops)
         self.rev.parent_ids = self.parents
         rev_tmp = StringIO()
         serializer_v5.write_revision(self.rev, rev_tmp)

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2005-10-14 01:56:08 +0000
+++ bzrlib/osutils.py	2005-10-15 11:38:29 +0000
@@ -24,6 +24,7 @@
 import os
 import re
 import sha
+import string
 import sys
 import time
 import types
@@ -418,3 +419,21 @@
         return True
     else:
         return False
+        
+
+def contains_whitespace(s):
+    """True if there are any whitespace characters in s."""
+    for ch in string.whitespace:
+        if ch in s:
+            return True
+    else:
+        return False
+
+
+def contains_linebreaks(s):
+    """True if there is any vertical whitespace in s."""
+    for ch in '\f\n\r':
+        if ch in s:
+            return True
+    else:
+        return False

=== modified file 'bzrlib/revision.py'
--- bzrlib/revision.py	2005-10-13 04:32:44 +0000
+++ bzrlib/revision.py	2005-10-14 04:32:41 +0000
@@ -14,9 +14,12 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+# TODO: Some kind of command-line display of revision properties: 
+# perhaps show them in log -v and allow them as options to the commit command.
 
 import bzrlib.errors
 from bzrlib.graph import node_distances, select_farthest, all_descendants
+from bzrlib.osutils import contains_whitespace
 
 NULL_REVISION="null:"
 
@@ -31,10 +34,17 @@
 
     parent_ids
         List of parent revision_ids
+
+    properties
+        Dictionary of revision properties.  These are attached to the
+        revision as extra metadata.  The name must be a single 
+        word; the value can be an arbitrary string.
     """
     
-    def __init__(self, revision_id, **args):
+    def __init__(self, revision_id, properties=None, **args):
         self.revision_id = revision_id
+        self.properties = properties or {}
+        self._check_properties()
         self.__dict__.update(args)
         self.parent_ids = []
         self.parent_sha1s = []
@@ -52,11 +62,22 @@
                 and self.timestamp == other.timestamp
                 and self.message == other.message
                 and self.timezone == other.timezone
-                and self.committer == other.committer)
+                and self.committer == other.committer
+                and self.properties == other.properties)
 
     def __ne__(self, other):
         return not self.__eq__(other)
 
+    def _check_properties(self):
+        """Verify that all revision properties are OK.
+        """
+        for name, value in self.properties.iteritems():
+            if not isinstance(name, basestring) or contains_whitespace(name):
+                raise ValueError("invalid property name %r" % name)
+            if not isinstance(value, basestring):
+                raise ValueError("invalid property value %r for %r" % 
+                                 (name, value))
+
 
 def is_ancestor(revision_id, candidate_id, branch):
     """Return true if candidate_id is an ancestor of revision_id.

=== modified file 'bzrlib/selftest/__init__.py'
--- bzrlib/selftest/__init__.py	2005-10-13 06:33:22 +0000
+++ bzrlib/selftest/__init__.py	2005-10-13 07:30:57 +0000
@@ -513,6 +513,7 @@
                    'bzrlib.selftest.test_conflicts',
                    'bzrlib.selftest.testtestament',
                    'bzrlib.selftest.testannotate',
+                   'bzrlib.selftest.testrevprops',
                    ]
 
     for m in (bzrlib.store, bzrlib.inventory, bzrlib.branch,

=== added file 'bzrlib/selftest/testrevprops.py'
--- bzrlib/selftest/testrevprops.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testrevprops.py	2005-10-14 04:26:55 +0000
@@ -0,0 +1,35 @@
+# (C) 2005 Canonical
+
+"""Tests for revision properties."""
+
+from bzrlib.branch import Branch
+from bzrlib.selftest import TestCaseInTempDir
+
+class TestRevProps(TestCaseInTempDir):
+    def test_simple_revprops(self):
+        """Simple revision properties"""
+        b = Branch.initialize('.')
+        props = dict(flavor='choc-mint', 
+                     condiment='orange\n  mint\n\tcandy')
+        b.commit(message='initial null commit', 
+                 revprops=props,
+                 allow_pointless=True,
+                 rev_id='test@user-1')
+        rev = b.get_revision('test@user-1')
+        self.assertTrue('flavor' in rev.properties)
+        self.assertEquals(rev.properties['flavor'], 'choc-mint')
+        self.assertEquals(sorted(rev.properties.items()),
+                          [('condiment', 'orange\n  mint\n\tcandy'),
+                           ('flavor', 'choc-mint')])
+
+    def test_invalid_revprops(self):
+        """Invalid revision properties"""
+        b = Branch.initialize('.')
+        self.assertRaises(ValueError,
+                          b.commit, 
+                          message='invalid',
+                          revprops={'what a silly property': 'fine'})
+        self.assertRaises(ValueError,
+                          b.commit, 
+                          message='invalid',
+                          revprops=dict(number=13))

=== modified file 'bzrlib/testament.py'
--- bzrlib/testament.py	2005-10-13 03:23:22 +0000
+++ bzrlib/testament.py	2005-10-14 04:23:13 +0000
@@ -73,25 +73,8 @@
 import string
 from sha import sha
 
-
-def contains_whitespace(s):
-    """True if there are any whitespace characters in s."""
-    for ch in string.whitespace:
-        if ch in s:
-            return True
-    else:
-        return False
-
-
-def contains_linebreaks(s):
-    """True if there is any vertical whitespace in s."""
-    for ch in '\f\n\r':
-        if ch in s:
-            return True
-    else:
-        return False
-
-    
+from bzrlib.osutils import contains_whitespace, contains_linebreaks
+
 class Testament(object):
     """Reduced summary of a revision.
 

=== modified file 'bzrlib/xml5.py'
--- bzrlib/xml5.py	2005-10-06 06:31:16 +0000
+++ bzrlib/xml5.py	2005-10-14 03:33:26 +0000
@@ -22,9 +22,6 @@
 from bzrlib.errors import BzrError
 
 
-
-
-
 class Serializer_v5(Serializer):
     """Version 5 serializer
 
@@ -99,8 +96,22 @@
                 p = SubElement(pelts, 'revision_ref')
                 p.tail = '\n'
                 p.set('revision_id', parent_id)
+        if rev.properties:
+            self._pack_revision_properties(rev, root)
         return root
-    
+
+
+    def _pack_revision_properties(self, rev, under_element):
+        top_elt = SubElement(under_element, 'properties')
+        for prop_name, prop_value in sorted(rev.properties.items()):
+            assert isinstance(prop_name, basestring) 
+            assert isinstance(prop_value, basestring) 
+            prop_elt = SubElement(top_elt, 'property')
+            prop_elt.set('name', prop_name)
+            prop_elt.text = prop_value
+            prop_elt.tail = '\n'
+        top_elt.tail = '\n'
+
 
     def _unpack_inventory(self, elt):
         """Construct from XML Element
@@ -173,12 +184,27 @@
             assert p.tag == 'revision_ref', \
                    "bad parent node tag %r" % p.tag
             rev.parent_ids.append(p.get('revision_id'))
-
+        self._unpack_revision_properties(elt, rev)
         v = elt.get('timezone')
         rev.timezone = v and int(v)
-
         rev.message = elt.findtext('message') # text of <message>
         return rev
 
 
+    def _unpack_revision_properties(self, elt, rev):
+        """Unpack properties onto a revision."""
+        props_elt = elt.find('properties')
+        assert len(rev.properties) == 0
+        if not props_elt:
+            return
+        for prop_elt in props_elt:
+            assert prop_elt.tag == 'property', \
+                "bad tag under properties list: %r" % p.tag
+            name = prop_elt.get('name')
+            value = prop_elt.text
+            assert name not in rev.properties, \
+                "repeated property %r" % p.name
+            rev.properties[name] = value
+
+
 serializer_v5 = Serializer_v5()

